// Doc.h


#pragma once

#include "Node.h"

typedef CList<CRect> CLstError;



enum
{
	maskT = 1<<0,
	maskR = 1<<1,
	maskB = 1<<2,
	maskL = 1<<3,
	maskWalls = maskT | maskR | maskB | maskL,
	
	// The limits between nodes (interior border)
	maskLimitT = 1<<4,
	maskLimitR = 1<<5,
	maskLimitB = 1<<6,
	maskLimitL = 1<<7,
	maskLimitFirst = maskLimitT,
	
	// The small black points in the corner
	maskTR = 1<< 8,
	maskBR = 1<< 9,
	maskBL = 1<<10,
	maskTL = 1<<11,
	maskPointFirst = maskTR,
	maskPoints = maskTR | maskBR | maskBL | maskTL,
	
	maskRoad = 1<<12,
	
	maskNothing = 1<<13,   // To indicate NOT to add a CELL
};


struct CELL
{
	CNode *pNode;
	int    iWall;   // Bits field (mask*)
	
	inline NODESEL GetSel() const
	{
		return (pNode) ? pNode->m_sel : selNo;
	}
	
	inline bool IsRoad() const
	{
		return ToBool(iWall & maskRoad);
	}
};


// Round the orientation (4 modulo)
inline int RoundOrient(int iOrient)
{
	return iOrient & 3;
}



extern const CSize aoffDir    [8];
extern const CSize aoffDiagDir[4];

// Borders coordinates (in cells) for each orientation
extern const CRect arcBorder[4];



class CDoc : public CDocument
{
protected:  // Creation
	
	DECLARE_DYNCREATE(CDoc)
	CDoc();
	
	
public:  // Data
	
	CSize    m_size;       // Board size in cells
	CLstNode m_lstNode;
	
	CLstError m_lstError;
	
	CELL *m_aCell;
	
	// Access
	CELL& GetCell(int x, int y)
	{
		ASSERT(m_aCell);
		return m_aCell[x + y*m_size.cx];
	}
	const CELL& GetCell(int x, int y) const
	{
		ASSERT(m_aCell);
		return m_aCell[x + y*m_size.cx];
	}
	inline CELL& GetCell(const CPoint& pt)
	{
		return GetCell(pt.x, pt.y);
	}
	inline const CELL& GetCell(const CPoint& pt) const
	{
		return GetCell(pt.x, pt.y);
	}
	inline CELL* GetSafeCell(const CPoint& pt)
	{
		return (IsPointInBoard(pt))
			? &GetCell(pt)
			: NULL;
	}
	
	
	CRect GetBoardRect() const
	{
		return CRect(CPoint(0,0), m_size);
	}
	
	
	// Indicate if a point is in the board
	bool IsPointInBoard(const CPoint pt) const
	{
		return ToBool(GetBoardRect().PtInRect(pt));
	}
	
	
	void SetAllSelect(NODESEL sel);
	
	
public:  // Functions
	
	virtual ~CDoc();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif
	
	bool IsOverlapOK(const CNode& node, bool fIgnoreSel=false) const;
	
	void AddNode(const CNode& node);
	
	void OnUpdateBoardSize();
	void OnSelectionChanged();
	
	void UpdateCells(bool fDocModified=true);
	void FindErrors();
	
	
public:  // Messages
	
	virtual void DeleteContents();
	virtual void Serialize(CArchive& ar);
	afx_msg void OnEditClear();
	afx_msg void OnUpdateNeedSel(CCmdUI* pCmdUI);
	afx_msg void OnGraphTool(UINT id);
	afx_msg void OnUpdateGraphTool(CCmdUI* pCmdUI);
	afx_msg void OnEditSelectAll();
	afx_msg void OnUpdateEditSelectAll(CCmdUI* pCmdUI);
	afx_msg void OnEditUnselect();
	afx_msg void OnGraphSize();
	DECLARE_MESSAGE_MAP()
};
